<?php
/*
	Plugin Name: Remove CPT base
	Description: Remove custom post type base slug from permalink url
	Version: 1.1
	Author: KubiQ
	Author URI: https://kubiq.sk
*/

if( ! class_exists('remove_cpt_base') ){
	class remove_cpt_base {
		var $domain = 'remove_cpt_base';
		var $plugin_admin_page;
		var $rcptb_selected;
		var $rcptb_prevent_duplicates;

		static public function flush_rewrite_rules(){
			$this->rewrite_rules(true);	
			$this->add_rules_htaccess();
		}

		function __construct(){
			$this->rewrite_rules();

			$mo = plugin_dir_path(__FILE__) . 'languages/' . get_locale() . '.mo';
			load_textdomain($this->domain, $mo);

			$this->rcptb_selected = get_option( 'rcptb_selected' );
			if( ! is_array( $this->rcptb_selected ) ) $this->rcptb_selected = array();
			$this->rcptb_prevent_duplicates = get_option( 'rcptb_prevent_duplicates' );

			add_action( 'admin_menu', array( &$this, 'plugin_menu_link' ) );
			add_action( 'wp_insert_post', array( &$this, 'post_save' ), 10, 1 );
			add_filter( 'post_type_link', array( &$this, 'remove_slug' ), 10, 3 );
			if( $this->rcptb_prevent_duplicates ){
				add_filter( 'wp_unique_post_slug', array( &$this, 'prevent_slug_duplicates' ), 10, 6 );
			}
		}

		function prevent_slug_duplicates( $slug, $post_ID, $post_status, $post_type, $post_parent, $original_slug ){
			if( ! in_array( $post_type, array('post', 'page') ) && ! in_array( $post_type, $this->rcptb_selected ) ){
				return $slug;
			}
			$post_match = get_page_by_path( $slug, 'OBJECT', 'post' );
			$page_match = get_page_by_path( $slug, 'OBJECT', 'page' );
			$custom_post_type_match = false;
			foreach( $this->rcptb_selected as $cpt ){
				if( get_page_by_path( $slug, 'OBJECT', $cpt ) ){
					$custom_post_type_match = true;
				}
			}
			if( $post_match || $page_match || $custom_post_type_match ){
				$slug .= '-duplicate';
			}
			return $slug;
		}
		
		function filter_plugin_actions($links, $file) {
		   $settings_link = '<a href="options-general.php?page=' . basename(__FILE__) . '">' . __('Settings') . '</a>';
		   array_unshift( $links, $settings_link );
		   return $links;
		}

		function post_save( $post_id ){
			global $wp_post_types;
			$post_type = get_post_type($post_id);
			foreach( $wp_post_types as $type => $custom_post ){
				if( $custom_post->_builtin == false && $type == $post_type && in_array( $custom_post->name, $this->rcptb_selected ) ){
					$this->rewrite_rules( true );
					$this->add_rules_htaccess();
					flush_rewrite_rules();
				}
			}
		}
		
		function plugin_menu_link() {
			$this->plugin_admin_page = add_submenu_page(
				'options-general.php',
				__( 'Remove CPT base', $this->domain ),
				__( 'Remove CPT base', $this->domain ),
				'manage_options',
				basename(__FILE__),
				array( $this, 'admin_options_page' )
			);
			add_filter( 'plugin_action_links_' . plugin_basename(__FILE__), array($this, 'filter_plugin_actions'), 10, 2 );
		}

		function admin_options_page(){
			if ( get_current_screen()->id != $this->plugin_admin_page ) return;
			global $wp_post_types; ?>
			<div class="wrap">
				<h2><?php _e( 'Remove base slug from url for these custom post types:', $this->domain ) ?></h2>
				<div class="below-h2 error"><p><?php _e( 'Because of WordPress limitations, you have to resave this page after each permalink structure update.' ) ?></p></div><?php
				if( isset( $_POST['rcptb_selected_sent'] ) ){
					$this->rcptb_prevent_duplicates = isset( $_POST['rcptb_prevent_duplicates'] ) ? intval( esc_attr( $_POST['rcptb_prevent_duplicates'] ) ) : 0;
					update_option( 'rcptb_prevent_duplicates', $this->rcptb_prevent_duplicates, 'no' );
					if( isset( $_POST['rcptb_selected'] ) && is_array( $_POST['rcptb_selected'] ) ){
						$this->rcptb_selected = array_map( 'esc_attr', (array)$_POST['rcptb_selected'] );
					}else{
						$this->rcptb_selected = array();
					}
					update_option( 'rcptb_selected', $this->rcptb_selected, 'no' );
					echo '<div class="below-h2 updated"><p>'.__( 'Settings saved.' ).'</p></div>';
					$this->rewrite_rules(true);
					$this->add_rules_htaccess();
					flush_rewrite_rules();
				} ?>
				<br>
				<form method="POST" action="">
					<input type="hidden" name="rcptb_selected_sent" value="1"><?php
					foreach( $wp_post_types as $type => $custom_post ){
						if( $custom_post->_builtin == false ){ ?>
							<div>
								<label>
									<input type="checkbox" name="rcptb_selected[]" value="<?php echo $custom_post->name ?>" <?php echo in_array( $custom_post->name, $this->rcptb_selected ) ? 'checked' : '' ?>>
									<?php echo $custom_post->label ?> (<?php echo $custom_post->name ?>)
								</label>
							</div><?php
						}
					} ?>
					<hr>
					<label>
						<input type="checkbox" name="rcptb_prevent_duplicates" value="1" <?php echo $this->rcptb_prevent_duplicates ? 'checked' : '' ?>>
						<?php _e( 'prevent slug duplicates for posts, pages and selected custom post types', $this->domain ) ?> <small>(<?php _e( 'this will add "-duplicate" to slug, in case of conflict', $this->domain ) ?>)</small>
					</label>
					<p class="submit">
						<input type="submit" class="button-primary" value="<?php _e( 'Save' ) ?>">
					</p>
				</form>
			</div><?php
		}

		function remove_slug( $permalink, $post, $leavename ){
			global $wp_post_types;
			foreach( $wp_post_types as $type => $custom_post ){
				if( $custom_post->_builtin == false && $type == $post->post_type && in_array( $custom_post->name, $this->rcptb_selected ) ){
					$custom_post->rewrite['slug'] = trim( $custom_post->rewrite['slug'], '/' );
					$permalink = str_replace( get_bloginfo('url') . '/' . $custom_post->rewrite['slug'] . '/', get_bloginfo('url') . "/", $permalink );
				}
			}
			return $permalink;
		}
		
		function rewrite_rules( $flash = false ){
			global $wp_post_types, $wpdb;
			$rcptb_selected = get_option( 'rcptb_selected' );
			if( ! is_array( $rcptb_selected ) ) $rcptb_selected = array();
			foreach( $wp_post_types as $type => $custom_post ){
				if( $custom_post->_builtin == false && in_array( $custom_post->name, $rcptb_selected ) ){
					$querystr = "SELECT {$wpdb->posts}.post_name FROM {$wpdb->posts} WHERE {$wpdb->posts}.post_status = 'publish'AND {$wpdb->posts}.post_type = '{$type}'";
					$posts = $wpdb->get_results( $querystr, OBJECT );
					foreach( $posts as $post ){
						$regex = "{$post->post_name}\$";
						add_rewrite_rule($regex, "index.php?{$custom_post->query_var}={$post->post_name}", 'top');			
					}
				}
			}
			if( $flash == true ){
				flush_rewrite_rules( false );
			}
		}

		function generate_rewrite_rules(){
			global $wp_post_types;
			$RewriteRules = array();
			foreach( $wp_post_types as $cpt ){
				if( $cpt->_builtin == false && in_array( $cpt->name, $this->rcptb_selected ) ){
					$RewriteRules[] = "RewriteRule ^{$cpt->query_var}/(.+)/\$ /\$1 [R=301,L]";
				}
			}
			return $RewriteRules;
		}

		function add_rules_htaccess(){
			$RewriteRules = $this->generate_rewrite_rules();
			$htaccess_filename = ABSPATH . '/.htaccess';
			if( is_readable( $htaccess_filename ) && is_writable( $htaccess_filename ) ){
				$htaccess = fopen( $htaccess_filename, 'r' );
				$content = fread( $htaccess, filesize( $htaccess_filename ) );
				fclose( $htaccess );
				$htaccess_tag = "REMOVE CPT BASE";
				$new_rules = '# BEGIN ' . $htaccess_tag . PHP_EOL;
				$new_rules .= str_replace( '$', '\\$', implode( PHP_EOL, $RewriteRules ) ) . PHP_EOL;
				$new_rules .= '# END ' . $htaccess_tag;
				if( strpos( $content, "# BEGIN {$htaccess_tag}") === false ){
					file_put_contents( $htaccess_filename, $new_rules . PHP_EOL . PHP_EOL . $content );
				}else{
					$pattern = "/# BEGIN {$htaccess_tag}.*?# END {$htaccess_tag}/ims"; 
					$content = preg_replace( $pattern, $new_rules, $content );
					file_put_contents( $htaccess_filename, $content );
				}
			}else{
				add_action( 'admin_notices', array( &$this, 'compatibility_notice' ) );
			}
		}
		
		function compatibility_notice(){
			$RewriteRules = $this->generate_rewrite_rules();
			echo '<div class="error fade">' . __( '.htaccess is not writable, please add following lines at the beginning manually', $this->domain ) . ':<br>' . implode( "<br>", $RewriteRules ) . '</div>';
		}
	}


	function rcptb_remove_plugin_options(){
		delete_option( 'rcptb_selected' );
		delete_option( 'rcptb_prevent_duplicates' );
	}

	$remove_cpt_base_var = new remove_cpt_base();
	register_activation_hook( __FILE__, array( $remove_cpt_base_var, 'flush_rewrite_rules' ) );
	register_deactivation_hook( __FILE__, array( $remove_cpt_base_var, 'flush_rewrite_rules' ) );
	register_uninstall_hook( __FILE__, 'rcptb_remove_plugin_options' );
}